/* -*-	Mode:C++; c-basic-offset:8; tab-width:8; indent-tabs-mode:t -*- */
/*
 * Copyright (c) 1997 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the Computer Systems
 *	Engineering Group at Lawrence Berkeley Laboratory.
 * 4. Neither the name of the University nor of the Laboratory may be used
 *    to endorse or promote products derived from this software without
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
/* Ported from CMU/Monarch's code, nov'98 -Padma.*/

/***************************************************************************
*	Modified by: Miguel Luís			September 2010						*
* 					nml17338@fct.unl.pt										*
* 				 Rodolfo Oliveira		September 2010						*
* 					rado@uninova.pt											*
* 																			*
* 	A novel distributed throughput-fairness optimal MAC protocol for ad hoc	*
* 	networks, motivated by the shared view of the channel.					*
* 																			*
*Faculty of Sciences and Technology (FCT)- Universidade Nova de Lisboa (UNL)*
 ***************************************************************************/


#ifndef __mac_timers_miop_h__
#define __mac_timers_miop_h__

/* ======================================================================
   Timers
   ====================================================================== */
class Mac_miop;

class MacTimer_miop : public Handler {
public:
	MacTimer_miop(Mac_miop* m) : mac(m) {
		busy_ = paused_ = 0; stime = rtime = 0.0; cw_avg_size = 0; cw_times = 1; paused_before = false;
		samples = 1000; slots_bsy = 0; slots_sum = 0; slots_t0 = 0; slots_t1 = 0; prob_idle_estim = 1;
		slots_busy= 0; first_backoff = true; accept_rxstate = false; rx_state_aux = MAC_IDLE;
		slots_succ = 0; slots_coll = 0; total_slots = 0;
	}

	virtual void handle(Event *e) = 0;

	virtual void start(double time);
	virtual void stop(void);
	virtual void pause(void) { assert(0); }
	virtual void resume(void) { assert(0); }

	inline int busy(void) { return busy_; }
	inline int paused(void) { return paused_; }

	inline double time1(void) { return time1_; };	// FSE
	inline double time2(void) { return time2_; };	// FSE
	inline double fs(void) { return fs_; }			// FSE

	inline double expire(void) {
		return ((stime + rtime) - Scheduler::instance().clock());
	}

protected:
	Mac_miop	*mac;
	int		busy_;
	int		paused_;
	Event		intr;
	double		stime;	// start time
	double		rtime;	// remaining time

	// Modified ML
	bool		paused_before;	// Calc Pi
	double 		slots_bsy;		// Calc Pi
	double 		slots_sum;		// Calc Pi
	double 		slots_t0;		// Calc Pi
	double 		slots_t1;		// Calc Pi
	double 		samples;		// Calc Pi

	double  	time1_;			// FSE
	double  	time2_;			// FSE
	double  	fs_;			// FSE

public:
	double  	numslots_;
	bool 		first;
	double 		cw_avg_size; 	// Stats
	double  	cw_times; 		// Stats
	double		slots_busy;

	double 		prob_idle_estim;
	bool 		first_backoff;

	double		slots_succ;		// Calc Ps
	double		slots_coll;		// Calc Ps
	bool		accept_rxstate;	// Calc Ps
	MacState	rx_state_aux;	// Calc Ps
	double		total_slots; 	// Calc Ps

	//
};


class BackoffTimer_miop : public MacTimer_miop {
public:
	BackoffTimer_miop(Mac_miop *m) : MacTimer_miop(m), difs_wait(0.0) {}

	void	start(int cw, int idle, double difs = 0.0);
	void	handle(Event *e);
	void	pause(void);
	void	resume(double difs);
private:
	double	difs_wait;
};

class DeferTimer_miop : public MacTimer_miop {
public:
	DeferTimer_miop(Mac_miop *m) : MacTimer_miop(m) {}

	void	start(double);
	void	handle(Event *e);
};

class BeaconTimer_miop : public MacTimer_miop {
public:
	BeaconTimer_miop(Mac_miop *m) : MacTimer_miop(m) {}

	void	start(double);
	void	handle(Event *e);
};

class ProbeTimer_miop : public MacTimer_miop {
public:
	ProbeTimer_miop(Mac_miop *m) : MacTimer_miop(m) {}

	void	start(double);
	void	handle(Event *e);
};


class IFTimer_miop : public MacTimer_miop {
public:
	IFTimer_miop(Mac_miop *m) : MacTimer_miop(m) {}

	void	handle(Event *e);
};

class NavTimer_miop : public MacTimer_miop {
public:
	NavTimer_miop(Mac_miop *m) : MacTimer_miop(m) {}

	void	handle(Event *e);
};

class RxTimer_miop : public MacTimer_miop {
public:
	RxTimer_miop(Mac_miop *m) : MacTimer_miop(m) {}

	void	handle(Event *e);
};

class TxTimer_miop : public MacTimer_miop {
public:
	TxTimer_miop(Mac_miop *m) : MacTimer_miop(m) {}

	void	handle(Event *e);
};



#endif /* __mac_timers_h__ */

